/*  bivbx.h

    Defines structs and functions for dealing with VBX controls.

*/
/*
 *      C/C++ Run Time Library - Version 6.0
 *
 *      Copyright (c) 1993 by Borland International
 *      All Rights Reserved.
 *
 */

#ifndef __BIVBX_H
#define __BIVBX_H

#define STRICT
#include <windows.h>

#ifdef __cplusplus
  extern "C" {            /* Assume C declarations for C++ */
#endif

#ifndef __VBX_H

  typedef short          SHORT;
  typedef unsigned short USHORT;
  typedef USHORT         ERR;
  typedef LPVOID         HCTL;
  typedef WORD           HPIC;
  typedef LPSTR          HSZ;
  typedef LPVOID         HLSTR;
  typedef unsigned char  ENUM;

  #ifdef RC_INVOKED
    #define DLGINIT 240
  #else
    #define RT_DLGINIT MAKEINTRESOURCE(240)
  #endif

#endif

/* VBX Window Messages */
#define WM_VBXFIREEVENT 0x0360

/* VBX Event Structure */
typedef struct VBXEVENT
{
  HCTL    Control;
  HWND    Window;
  int     ID;
  int     EventIndex;
  LPCSTR  EventName;
  int     NumParams;
  LPVOID  ParamList;

} VBXEVENT;

/* macros for accessing event arguments */
#define VBX_EVENTARGNUM(event,type,index) \
  *(((type FAR * FAR *)event->ParamList)[event->NumParams - index - 1])
#define VBX_EVENTARGSTR(event,index) \
  (((HLSTR FAR *)event->ParamList)[event->NumParams - index - 1])
  
/* Initialization */
BOOL   FAR _export PASCAL VBXInit( HINSTANCE hInst );
void   FAR _export PASCAL VBXTerm( void );

/* Controls */
HCTL   FAR _export PASCAL VBXGetHctl( HWND hWnd );
HWND   FAR _export PASCAL VBXGetHwnd( HCTL hCtl );
HCTL   FAR _export PASCAL VBXCreate( HWND hWndParent, UINT ID, LPCSTR lpszDLL,
                                     LPCSTR lpszClass, LPCSTR lpszTitle,
                                     DWORD dwStyle, int x, int y,
                                     int cx, int cy, int hFile );

/* Dialogs */
BOOL   FAR _export PASCAL VBXInitDialog( HWND hWnd, HINSTANCE hInst, LPSTR lpID );

/* Properties */
int    FAR _export PASCAL VBXGetNumProps( HCTL hCtl );
BOOL   FAR _export PASCAL VBXGetProp( HCTL hCtl, int iProp, LPVOID lpValue );
ERR    FAR _export PASCAL VBXGetPropByName( HCTL hCtl, LPSTR lpszName, LPVOID lpValue );
int    FAR _export PASCAL VBXGetPropIndex( HCTL hCtl, LPCSTR lpszName );
LPCSTR FAR _export PASCAL VBXGetPropName( HCTL hCtl, int iProp );
BOOL   FAR _export PASCAL VBXSetProp( HCTL hCtl, int iProp, LONG lValue );
ERR    FAR _export PASCAL VBXSetPropByName( HCTL hCtl, LPSTR lpszName, LONG lVal );

/* Events */
int    FAR _export PASCAL VBXGetEventIndex( HCTL hCtl, LPCSTR lpszName );
LPCSTR FAR _export PASCAL VBXGetEventName( HCTL hCtl, int iEvent );
int    FAR _export PASCAL VBXGetNumEvents( HCTL hCtl );

/* methods */
BOOL   FAR _export PASCAL VBXMethod( HCTL hCtl, int iMethod, long far * lpArgs );
BOOL   FAR _export PASCAL VBXMethodAddItem( HCTL hCtl, int iItem, LPCSTR item );
BOOL   FAR _export PASCAL VBXMethodDrag( HCTL hCtl, int iAction );
BOOL   FAR _export PASCAL VBXMethodMove( HCTL hCtl, int x, int y, int w, int h );
BOOL   FAR _export PASCAL VBXMethodRefresh( HCTL hCtl );
BOOL   FAR _export PASCAL VBXMethodRemoveItem( HCTL hCtl, int iItem );

#ifndef __VBX_H

/* dynamic strings (HSZ) */
HSZ    FAR _export PASCAL VBCreateHsz(HANDLE seg, LPSTR lpszString);
LPSTR  FAR _export PASCAL VBDerefHsz(HSZ hsz);
HSZ    FAR _export PASCAL VBDestroyHsz(HSZ hsz);
LPSTR  FAR _export PASCAL VBLockHsz(HSZ hsz);
VOID   FAR _export PASCAL VBUnlockHsz(HSZ hsz);

/* Basic language strings (HLSTR) */
HLSTR  FAR _export PASCAL VBCreateHlstr(LPVOID pb, USHORT cbLen);
LPSTR  FAR _export PASCAL VBDerefHlstr(HLSTR hlstr);
VOID   FAR _export PASCAL VBDestroyHlstr(HLSTR hlstr);
USHORT FAR _export PASCAL VBGetHlstrLen(HLSTR hlstr);
ERR    FAR _export PASCAL VBSetHlstr(HLSTR far *phlstr, LPVOID pb, USHORT cbLen);

/* pixel/twips conversions */
LONG   FAR _export PASCAL VBYPixelsToTwips(SHORT Pixels);
LONG   FAR _export PASCAL VBXPixelsToTwips(SHORT Pixels);
SHORT  FAR _export PASCAL VBYTwipsToPixels(LONG Twips);
SHORT  FAR _export PASCAL VBXTwipsToPixels(LONG Twips);

/* color properties */
#define COLOR_SYSCOLOR          0x80000000L
#define MAKESYSCOLOR(iColor)    ((COLORREF)(iColor|COLOR_SYSCOLOR))

/* pictures */
typedef struct tagPIC
{
    BYTE    picType;
    union
    {
	struct
	{
	  HBITMAP   hbitmap;		// bitmap
	  HPALETTE  hpal;		// accompanying palette
	} bmp;
	struct
	{
	  HANDLE    hmeta;		// Metafile
	  int	    xExt, yExt; 	// extent
	} wmf;
	struct
	{
	  HICON     hicon;		// Icon
	} icon;
    } picData;
    BYTE    picReserved[4];
} PIC;

typedef PIC FAR  *  LPPIC;

#define PICTYPE_NONE		0
#define PICTYPE_BITMAP		1
#define PICTYPE_METAFILE	2
#define PICTYPE_ICON		3

#define HPIC_INVALID		0xFFFF
#define HPIC_NULL		0

HPIC FAR _export PASCAL VBAllocPic(LPPIC lppic);
VOID FAR _export PASCAL VBFreePic(HPIC hpic);
HPIC FAR _export PASCAL VBGetPic(HPIC hpic, LPPIC lppic);
ERR  FAR _export PASCAL VBPicFromCF(HPIC FAR *lphpic, HANDLE hData, WORD wFormat);
HPIC FAR _export PASCAL VBRefPic(HPIC hpic);

/* array property structure */
typedef struct tagDATASTRUCT
{
    LONG   data;	    /* Data for Get/Set */
    USHORT cindex;	    /* Number of indecies */
    struct
    {
	USHORT datatype;    /* Type of nth index (Currently always DT_SHORT) */
	LONG   data;	    /* Value of nth index */

    } index[1]; 	    /* Currently, only 1-dim arrays supported */

} DATASTRUCT;

typedef DATASTRUCT FAR	*LPDATASTRUCT;
typedef DATASTRUCT NEAR *NPDATASTRUCT;

#endif // __VBX_H

#ifdef __cplusplus
  }
#endif

#endif  /* __BIVBX_H */

