/*------------------------------------------------------------------------*/
/*                                                                        */
/*  CSTRING.H                                                             */
/*                                                                        */
/*                                                                        */
/*------------------------------------------------------------------------*/

/*
 *      C/C++ Run Time Library - Version 6.0
 *
 *      Copyright (c) 1987, 1993 by Borland International
 *      All Rights Reserved.
 *
 */

#ifndef __cplusplus
#error Must use C++ for CSTRING.H
#endif

#ifndef __CSTRING_H
#define __CSTRING_H

/*------------------------------------------------------------------------*/
/*                                                                        */
/*  This class uses a technique called "copy-on-write".                   */
/*  Multiple instances of a string can refer to the same piece of data    */
/*  so long as it is in a "readonly" situation.  If a string writes to    */
/*  the data, then a copy is automatically made if more than one string   */
/*  is referring to it.                                                   */
/*                                                                        */
/*------------------------------------------------------------------------*/

#define STRICT

#if !defined(__WINDOWS_H)
#include <windows.h>
#endif

#if !defined(__REF_H)
#include <ref.h>
#endif

#if !defined(__STRING_H)
#include <string.h>
#endif

#if !defined(__CTYPE_H)
#include <ctype.h>
#endif

#if !defined(__STDDEF_H)
#include <stddef.h>
#endif

#if !defined(__EXCEPT_H)
#include <except.h>
#endif

class _EXPCLASS string;
class _EXPCLASS TRegexp;
class _EXPCLASS TSubString;
class _EXPCLASS TStringRef;
class _EXPCLASS istream;
class _EXPCLASS ostream;
class _EXPCLASS ipstream;
class _EXPCLASS opstream;

/*------------------------------------------------------------------------*/
/*                                                                        */
/*  Since inline functions that use throw specifiers                      */
/*  currently end up rather large, we don't use throw                     */
/*  specifiers on them unless you #define USE_THROW_SPECIFIERS.           */
/*                                                                        */
/*------------------------------------------------------------------------*/

#if defined( USE_THROW_SPECIFIERS )
#define THROW_NONE                  throw()
#define THROW_XALLOC                throw(xalloc)
#define THROW_OUTOFRANGE            throw( string::outofrange )
#define THROW_XALLOC_LENGTHERROR    throw( xalloc, string::lengtherror )
#define THROW_XALLOC_OUTOFRANGE     throw( xalloc, string::outofrange )
#define THROW_XALLOC_RANGE_LENGTH   \
        throw( xalloc, string::lengtherror, string::outofrange )
#else
#define THROW_NONE
#define THROW_XALLOC
#define THROW_OUTOFRANGE
#define THROW_XALLOC_LENGTHERROR
#define THROW_XALLOC_OUTOFRANGE
#define THROW_XALLOC_RANGE_LENGTH
#endif


/*------------------------------------------------------------------------*/
/*                                                                        */
/*  string                                                                */
/*                                                                        */
/*------------------------------------------------------------------------*/

const size_t NPOS = size_t(-1);

class _EXPCLASS string
{

public:

    //
    // Exceptions
    //
    class outofrange : public xmsg
    {
    public:
        outofrange();
    };

    class lengtherror : public xmsg
    {
    public:
        lengtherror();
    };

    //
    // Constructors
    //

    string() throw( xalloc );

    string( const string _FAR &s ) throw( xalloc );
    string( const string _FAR &s, size_t n ) throw( xalloc );

    string( const char _FAR *cp ) throw( xalloc, lengtherror );
    string(const char _FAR *cp, size_t n) throw( xalloc, lengtherror );

    string( char c ) THROW_XALLOC_LENGTHERROR;
    string( char c, size_t n ) THROW_XALLOC_LENGTHERROR;

    string( signed char c ) THROW_XALLOC_LENGTHERROR;
    string( signed char c, size_t n ) THROW_XALLOC_LENGTHERROR;

    string( unsigned char c ) THROW_XALLOC_LENGTHERROR;
    string( unsigned char c, size_t n ) THROW_XALLOC_LENGTHERROR;

    // non-standard constructors
    string( const TSubString _FAR &ss ) throw( xalloc );

    // Special far string ctors for small & medium model
    #if defined( __TINY__ ) || defined( __SMALL__ ) || defined( __MEDIUM__ )
    string( const char __far *cp ) THROW_XALLOC_LENGTHERROR;
    string( const char __far *cp, size_t n ) THROW_XALLOC_LENGTHERROR;
    #endif

    // Ctor to make a string from a resource
    #if defined( _Windows )
    string( HINSTANCE instance, UINT id, int len = 255 )
        throw( xalloc, lengtherror );
    #endif

    //
    // Destructor
    //
    ~string() throw();

    //
    // Assignment
    //
    string _FAR & operator = ( const string _FAR &s ) THROW_XALLOC;
    string _FAR & assign( const string _FAR &s ) THROW_XALLOC;
    string _FAR & assign( const string _FAR &s, size_t n ) throw( xalloc );

    //
    // Concatenation
    //
    string _FAR & operator += ( const string _FAR &s )
        THROW_XALLOC_LENGTHERROR;
    string _FAR & append( const string _FAR &s )
        THROW_XALLOC_LENGTHERROR;
    string _FAR & append( const string _FAR &s, size_t n )
        throw( xalloc, lengtherror );

    string _FAR & operator += ( const char _FAR *cp )
        THROW_XALLOC_LENGTHERROR;
    friend string _Cdecl _FARFUNC operator + ( const string _FAR &s,
                                               const char _FAR *cp )
        THROW_XALLOC_LENGTHERROR;
    string _FAR & append( const char _FAR *cp )
        throw( xalloc, lengtherror );
    string _FAR & append( const char _FAR *cp, size_t n )
        throw( xalloc, lengtherror );

    string _FAR &prepend( const string _FAR &s )
        THROW_XALLOC_LENGTHERROR;
    string _FAR &prepend( const string _FAR &s, size_t n )
        THROW_XALLOC_LENGTHERROR;
    string _FAR &prepend( const char _FAR *cp )
        THROW_XALLOC_LENGTHERROR;
    string _FAR &prepend( const char _FAR *cp, size_t n )
        throw( xalloc, lengtherror );

    //
    // Comparison
    //
    int compare(const string _FAR &s) const throw();
    int compare(const string _FAR &s, size_t n ) const throw();

    friend int operator == ( const string _FAR &s1, const string _FAR &s2 )
        THROW_NONE;

    friend int operator != ( const string _FAR &s1, const string _FAR &s2 )
        THROW_NONE;

    friend int operator == ( const string _FAR &s, const char _FAR *cp )
        THROW_NONE;
    friend int operator == ( const char _FAR *cp, const string _FAR &s )
        THROW_NONE;

    friend int operator != ( const string _FAR &s, const char _FAR *cp )
        THROW_NONE;
    friend int operator != ( const char _FAR *cp, const string _FAR &s )
        THROW_NONE;

    friend int operator <  ( const string _FAR &s1, const string _FAR &s2 )
        THROW_NONE;
    friend int operator <  ( const string _FAR &s, const char _FAR *cp )
        THROW_NONE;
    friend int operator <  ( const char _FAR *cp, const string _FAR &s )
        THROW_NONE;

    friend int operator <= ( const string _FAR &s1, const string _FAR &s2 )
        THROW_NONE;
    friend int operator <= ( const string _FAR &s, const char _FAR *cp )
        THROW_NONE;
    friend int operator <= ( const char _FAR *cp, const string _FAR &s )
        THROW_NONE;

    friend int operator >  ( const string _FAR &s1, const string _FAR &s2 )
        THROW_NONE;
    friend int operator >  ( const string _FAR &s, const char _FAR *cp )
        THROW_NONE;
    friend int operator >  ( const char _FAR *cp, const string _FAR &s )
        THROW_NONE;

    friend int operator >= ( const string _FAR &s1, const string _FAR &s2 )
        THROW_NONE;
    friend int operator >= ( const string _FAR &s, const char _FAR *cp )
        THROW_NONE;
    friend int operator >= ( const char _FAR *cp, const string _FAR &s )
        THROW_NONE;

    //
    // Insertion at some position
    //
    string _FAR &insert( size_t pos, const string _FAR &s )
        throw( xalloc, outofrange, lengtherror );
    string _FAR &insert( size_t pos, const string _FAR &s, size_t n )
        throw( xalloc, outofrange, lengtherror );

    //
    // Removal
    //
    string _FAR &remove( size_t pos ) THROW_XALLOC_OUTOFRANGE;
    string _FAR &remove( size_t pos, size_t n )
        throw( xalloc, outofrange );

    //
    // Replacement at some position
    //
    string _FAR &replace( size_t pos, size_t n, const string _FAR &s )
        THROW_XALLOC_RANGE_LENGTH;
    string _FAR &replace( size_t pos, size_t n1,
                          const string _FAR &s, size_t n2 )
        throw( xalloc, outofrange, lengtherror );

    //
    // Subscripting
    //
    char get_at( size_t pos ) const THROW_OUTOFRANGE;
    void put_at( size_t pos, char c ) THROW_OUTOFRANGE;

    char _FAR & operator[]( size_t pos ) THROW_OUTOFRANGE;
    char _FAR & operator()( size_t pos ) throw( outofrange );
    TSubString operator()( size_t start, size_t len ) THROW_NONE;
    TSubString operator()( const TRegexp _FAR &re ) THROW_NONE;
    TSubString operator()( const TRegexp _FAR &re, size_t start ) throw();

    char operator[]( size_t pos ) const THROW_OUTOFRANGE;
    char operator()( size_t pos ) const THROW_OUTOFRANGE;
    const TSubString operator()( size_t start, size_t len ) const throw();
    const TSubString operator()( const TRegexp _FAR &pat ) const THROW_NONE;
    const TSubString operator()( const TRegexp _FAR &pat, size_t start )
        const throw();

    //
    // Searching
    //
    size_t find( const string _FAR &s ) const THROW_NONE;
    size_t find( const string _FAR &s, size_t pos ) const throw();
    size_t rfind( const string _FAR &s ) const THROW_NONE;
    size_t rfind( const string _FAR &s, size_t pos ) const throw();

    int contains( const char _FAR *pat ) const throw();
    int contains(const string _FAR &s) const THROW_NONE;
    size_t find( const TRegexp _FAR &pat, size_t i = 0 ) const throw();
    size_t find( const TRegexp _FAR &pat, size_t _FAR *ext, size_t i = 0 )
        const throw();

    //
    // Substring
    //
    string substr( size_t pos ) const
        throw( xalloc, outofrange );
    string substr( size_t pos, size_t n ) const
        throw( xalloc, outofrange );

    TSubString substring( const char _FAR *cp ) THROW_NONE;
    const TSubString substring( const char _FAR *cp )
        const THROW_NONE;
    TSubString substring( const char _FAR *cp, size_t start ) throw();
    const TSubString substring( const char _FAR *cp, size_t start )
        const throw();

    //
    // Character set searching
    //
    size_t find_first_of( const string _FAR &s ) const THROW_NONE;
    size_t find_first_of( const string _FAR &s, size_t pos ) const throw();
    size_t find_first_not_of( const string _FAR &s ) const THROW_NONE;
    size_t find_first_not_of( const string _FAR &s, size_t pos )
        const throw();
    size_t find_last_of( const string _FAR &s ) const THROW_NONE;
    size_t find_last_of( const string _FAR &s, size_t pos ) const throw();
    size_t find_last_not_of( const string _FAR &s ) const THROW_NONE;
    size_t find_last_not_of( const string _FAR &s, size_t pos )
        const throw();

    //
    // Miscellaneous
    //
    size_t length() const THROW_NONE;
    size_t copy( char _FAR *cb, size_t n ) throw( outofrange );
    size_t copy( char _FAR *cb, size_t n, size_t pos ) throw( outofrange );
    const char _FAR *c_str() const THROW_XALLOC;
    size_t reserve() const THROW_NONE;
    void reserve( size_t ic ) throw( xalloc, outofrange );

    string copy() const throw( xalloc );    // Distinct copy of self.

    friend ipstream _FAR & _EXPFUNC operator >> ( ipstream _FAR & is,
                                         string _FAR & str );

    // Static member functions:
    static int set_case_sensitive( int tf = 1 );
    static int set_paranoid_check( int ck = 1 );
    static int skip_whitespace( int sk = 1 );
    static size_t initial_capacity( size_t ic = 63 );
    static size_t resize_increment( size_t ri = 64 );
    static size_t max_waste( size_t mw = 63 );

    static int get_case_sensitive_flag();
    static int get_paranoid_check_flag();
    static int get_skip_whitespace_flag();
    static size_t get_initial_capacity();
    static size_t get_resize_increment();
    static size_t get_max_waste();

    enum StripType { Leading, Trailing, Both };

    // Non-static member functions:
    unsigned hash() const;
    int is_null() const;
    istream _FAR &read_file( istream _FAR &is );
    istream _FAR &read_string( istream _FAR &is );
    istream _FAR &read_line( istream _FAR &is );
    istream _FAR &read_to_delim( istream _FAR &is, char delim = '\n' );
    istream _FAR &read_token( istream _FAR &is );
    void resize( size_t m );
    TSubString strip( StripType s = Trailing, char c = ' ' );
    void to_lower();
    void to_upper();

    #if defined( _Windows )
    void ansi_to_oem() THROW_NONE;
    void oem_to_ansi() THROW_NONE;
    #endif

protected:

    int  valid_element( size_t pos ) const THROW_NONE;
    int  valid_index( size_t pos ) const THROW_NONE;

    void assert_element( size_t pos ) const throw( outofrange );
    void assert_index( size_t pos ) const throw( outofrange );

    string( const string _FAR &s, const char _FAR *cb );
    void cow();

private:

    TStringRef _FAR *p;

    static int case_sensitive;
    static int paranoid_check;
    static int skip_white;
    static size_t initial_capac;
    static size_t resize_inc;
    static size_t freeboard;

private:

    friend class _EXPCLASS TSubString;
    friend class _EXPCLASS TStringRef;

    void clone();
    size_t find_case_index( const char _FAR *cb,
                            size_t start,
                            size_t _FAR &patl) const;
    size_t rfind_case_index( const char _FAR *cb,
                             size_t start,
                             size_t _FAR &patl) const;
    size_t find_index(const char _FAR *,
                      size_t start,
                      size_t _FAR & patl) const;
    size_t rfind_index(const char _FAR *,
                       size_t start,
                       size_t _FAR & patl) const;
    unsigned hash_case() const;

};

#if defined( BI_OLDNAMES )
#define TString string
#define BI_String string
#endif

/*------------------------------------------------------------------------*/
/*                                                                        */
/*  Related global functions                                              */
/*                                                                        */
/*------------------------------------------------------------------------*/

istream _FAR &
_Cdecl _FARFUNC operator >> ( istream _FAR &is, string _FAR &s );

ostream _FAR &
_Cdecl _FARFUNC operator << ( ostream _FAR &os, const string _FAR &s );

istream _FAR &
_Cdecl getline( istream _FAR &is, string _FAR &s );

istream _FAR &
_Cdecl getline( istream _FAR &is, string _FAR &s, char c );

opstream _FAR&
_Cdecl _FARFUNC operator << ( opstream _FAR & os, const string _FAR & str );

string _Cdecl _FARFUNC to_lower( const string _FAR &s ) throw();
string _Cdecl _FARFUNC to_upper( const string _FAR &s ) throw();
string _Cdecl _FARFUNC operator + ( const char _FAR *cp,
                                    const string _FAR & s)
                                    throw( xalloc, string::lengtherror );
string _Cdecl _FARFUNC operator + ( const string _FAR &s1,
                                    const string _FAR &s2 )
                                    THROW_XALLOC_LENGTHERROR;

/*------------------------------------------------------------------------*/
/*                                                                        */
/*  TStringRef                                                            */
/*                                                                        */
/*  This is the dynamically allocated part of a string.                   */
/*  It maintains a reference count.                                       */
/*  There are no public member functions.                                 */
/*                                                                        */
/*------------------------------------------------------------------------*/

class _EXPCLASS TStringRef : public TReference
{

    friend class _EXPCLASS string;
    friend class _EXPCLASS TSubString;

    //
    // Data
    //
    char _FAR *array;
    size_t nchars;
    size_t capacity;

    //
    // Constructors
    //
    TStringRef( char c, size_t n );
    TStringRef( const char _FAR *str1, size_t count1,
                const char _FAR *str2, size_t count2,
                size_t extra );

    // Special far TStringRef ctor for small data models
    #if defined( __TINY__ ) || defined( __SMALL__ ) || defined( __MEDIUM__ )
    TStringRef( const char __far*, size_t n = NPOS );
    #endif

    //
    // Ctor to make a TStringRef from a resource
    //
    #if defined( _Windows )
    TStringRef( HINSTANCE instance, UINT id, int len = 255 )
         throw( xalloc, string::lengtherror );
    #endif

    //
    // Destructor
    //
    ~TStringRef() throw();

    //
    // Miscellaneous
    //
    void reserve( size_t ic ) throw( xalloc, string::outofrange );
    void check_freeboard() throw();
    void grow_to( size_t n ) throw( xalloc, string::lengtherror );
    void read_to_delim( istream _FAR &is, char delim ) throw( xalloc );
    void read_token( istream _FAR &is ) throw( xalloc );
    static size_t round_capacity( size_t cap ) throw();
    void splice( size_t start, size_t extent,
                 const char _FAR *cp, size_t n )
        throw( xalloc, string::lengtherror );

    friend ipstream _FAR & _EXPFUNC operator >> ( ipstream _FAR & is,
                                         string _FAR & str );
};

#if defined( BI_OLDNAMES )
#define BI_StringRef TStringRef
#endif

/*------------------------------------------------------------------------*/
/*                                                                        */
/*  TSubString                                                            */
/*                                                                        */
/*  The TSubString class allows selected elements to be addressed.        */
/*  There are no public constructors.                                     */
/*                                                                        */
/*------------------------------------------------------------------------*/

class _EXPCLASS TSubString
{

public:

    //
    // Assignment
    //
    TSubString _FAR & operator = ( const string _FAR &s ) throw();

    //
    // Comparison
    //
    int operator == ( const char _FAR *cp ) const throw();
    int operator == ( const string _FAR &s ) const THROW_NONE;
    int operator != ( const char _FAR *cp ) const THROW_NONE;
    int operator != ( const string _FAR & str ) const THROW_NONE;

    //
    // Subscripting
    //
    char get_at( size_t pos ) const THROW_OUTOFRANGE;
    void put_at( size_t pos, char c ) THROW_OUTOFRANGE;

    char _FAR & operator[]( size_t pos ) THROW_OUTOFRANGE;
    char _FAR & operator()( size_t pos ) throw( string::outofrange );
    char operator[]( size_t pos ) const THROW_OUTOFRANGE;
    char operator()( size_t pos ) const THROW_OUTOFRANGE;
    size_t length() const THROW_NONE;
    int start() const THROW_NONE;
    void to_lower() throw();
    void to_upper() throw();

    //
    // Detecting empty strings
    //
    int is_null() const THROW_NONE;
    int operator!() const THROW_NONE;

protected:

    void assert_element( size_t pos ) const throw( string::outofrange );
    int valid_element( size_t pos ) const;

private:

    friend class _EXPCLASS string;

    //
    // Data
    //
    string _FAR *s;
    size_t begin;
    size_t extent;

    //
    // Constructor
    //
    TSubString( const string _FAR *cp, size_t start, size_t len );

};

#if defined( BI_OLDNAMES )
#define BI_SubString TSubString
#endif

/*------------------------------------------------------------------------*/
/*                                                                        */
/*  string inlines                                                        */
/*                                                                        */
/*------------------------------------------------------------------------*/

inline string::outofrange::outofrange() :
    xmsg( "String reference out of range" )
{
}

inline string::lengtherror::lengtherror() :
    xmsg( "String length error" )
{
}

inline string::string( char c ) THROW_XALLOC_LENGTHERROR
{
    p = new TStringRef(c,1);
}

inline string::string( char c, size_t n ) THROW_XALLOC_LENGTHERROR
{
    p = new TStringRef(c,n);
}

inline string::string( signed char c ) THROW_XALLOC_LENGTHERROR
{
    p = new TStringRef(c,1);
}

inline string::string( signed char c, size_t n ) THROW_XALLOC_LENGTHERROR
{
    p = new TStringRef(c,n);
}

inline string::string( unsigned char c ) THROW_XALLOC_LENGTHERROR
{
    p = new TStringRef(c,1);
}

inline string::string( unsigned char c, size_t n ) THROW_XALLOC_LENGTHERROR
{
    p = new TStringRef(c,n);
}

#if defined( __TINY__ ) || defined( __SMALL__ ) || defined( __MEDIUM__ )
// Far string ctors make a near string from a far string

inline string::string( const char __far *cp ) THROW_XALLOC_LENGTHERROR
{
    p = new TStringRef(cp);
}

inline string::string( const char __far *cp, size_t n )
    THROW_XALLOC_LENGTHERROR
{
    p = new TStringRef(cp,n);
}
#endif

inline string _FAR & string::operator = ( const string _FAR &s )
    THROW_NONE
{
    return assign( s, NPOS );
}

inline string _FAR & string::assign( const string _FAR &s )
    THROW_NONE
{
    return assign( s, NPOS );
}

inline string _FAR & string::operator += ( const string _FAR &s )
    THROW_XALLOC_LENGTHERROR
{
    return append(s,NPOS);
}

inline string _FAR & string::append( const string _FAR &s )
    THROW_XALLOC_LENGTHERROR
{
    return append(s,NPOS);
}

inline string _FAR & string::prepend( const char _FAR *cp )
    THROW_XALLOC_LENGTHERROR
{
    return prepend( cp, strlen(cp) );
}

inline int operator == ( const string _FAR &s1, const string _FAR &s2 )
    THROW_NONE
{
    return s1.compare( s2 ) == 0;
}

inline int operator != ( const string _FAR &s1, const string _FAR &s2 )
    THROW_NONE
{
    return !(s1==s2);
}

inline string _FAR &string::remove( size_t pos )
    THROW_XALLOC_OUTOFRANGE
{
    return remove( pos, length() );
}

inline string _FAR &string::replace( size_t pos,
                                     size_t n,
                                     const string _FAR &s )
    THROW_XALLOC_RANGE_LENGTH
{
    return replace( pos, n, s, NPOS );
}

inline char string::get_at( size_t pos ) const THROW_OUTOFRANGE
{
    return (*this)[pos];
}

inline void string::put_at( size_t pos, char c ) THROW_OUTOFRANGE
{
    (*this)[pos] = c;
}

inline char _FAR & string::operator[]( size_t pos ) THROW_OUTOFRANGE
{
    return (*this)(pos);    // use operator()
}

inline TSubString string::operator()( size_t start, size_t len ) THROW_NONE
{
    return TSubString( this, start, len );
}

inline size_t string::find( const string _FAR &s ) const THROW_NONE
{
    return find( s, 0 );
}

inline size_t string::rfind( const string _FAR &s ) const THROW_NONE
{
    return rfind( s, s.length() );
}

inline size_t string::length() const THROW_NONE
{
    return p->nchars;
}

inline const char _FAR *string::c_str() const THROW_NONE
{
    return p->array;
}

inline size_t string::reserve() const THROW_NONE
{
    return p->capacity;
}

inline void string::cow()
{
    if( p->References() > 1 )
        clone();
}

inline string _FAR &string::operator += ( const char _FAR *cp )
    THROW_XALLOC_LENGTHERROR
{
    return append( cp, strlen(cp) );
}

inline string _FAR &string::prepend( const string _FAR &s )
    THROW_XALLOC_LENGTHERROR
{
    return prepend( s.c_str() );
}

inline string _FAR &string::prepend( const string _FAR &s, size_t n )
    THROW_XALLOC_LENGTHERROR
{
    return prepend( s.c_str(), n );
}

inline int operator == ( const string _FAR &s1, const char _FAR *s2 ) THROW_NONE
{
    return s1.compare(s2) == 0;
}

inline int operator == ( const char _FAR *cp, const string _FAR &s ) THROW_NONE
{
    return string(cp).compare(s) == 0;
}

inline int operator != ( const string _FAR &s, const char _FAR *cp ) THROW_NONE
{
    return !(s==cp);
}

inline int operator != ( const char _FAR *cp, const string _FAR &s ) THROW_NONE
{
    return !(cp==s);
}

inline int operator <  ( const string _FAR &s1, const string _FAR &s2 )
    THROW_NONE
{
    return s1.compare(s2) < 0;
}

inline int operator <  ( const string _FAR &s1, const char _FAR *s2 ) THROW_NONE
{
    return s1.compare(s2) < 0;
}

inline int operator <  ( const char _FAR *cp, const string _FAR &s ) THROW_NONE
{
    return string(cp).compare(s) < 0;
}

inline int operator <= ( const string _FAR &s1, const string _FAR &s2 )
    THROW_NONE
{
    return s1.compare(s2) <= 0;
}

inline int operator <= ( const string _FAR &s, const char _FAR *cp ) THROW_NONE
{
    return s.compare(string(cp)) <= 0;
}

inline int operator <= ( const char _FAR *cp, const string _FAR &s ) THROW_NONE
{
    return string(cp).compare(s) <= 0;
}

inline int operator >  ( const string _FAR &s1, const string _FAR &s2 )
    THROW_NONE
{
    return s1.compare(s2) > 0;
}

inline int operator >  ( const string _FAR &s, const char _FAR *cp ) THROW_NONE
{
    return s.compare(cp) > 0;
}

inline int operator >  ( const char _FAR *cp, const string _FAR &s ) THROW_NONE
{
    return string(cp).compare(s) > 0;
}

inline int operator >= ( const string _FAR &s1, const string _FAR &s2 )
    THROW_NONE
{
    return s1.compare(s2) >= 0;
}

inline int operator >= ( const string _FAR &s, const char _FAR *cp ) THROW_NONE
{
    return s.compare(cp) >= 0;
}

inline int operator >= ( const char _FAR *cp, const string _FAR &s ) THROW_NONE
{
    return string(cp).compare(s) >= 0;
}

inline char string::operator[]( size_t pos ) const THROW_OUTOFRANGE
{
    assert_element(pos);
    return p->array[pos];
}

inline char string::operator()( size_t pos ) const THROW_OUTOFRANGE
{
#if defined( BOUNDS_CHECK )
    assert_element(pos);
#endif
    return p->array[pos];
}

inline int string::contains( const string _FAR &s ) const THROW_NONE
{
    return contains( s.c_str() );
}

inline TSubString string::substring( const char _FAR *cp ) THROW_NONE
{
    return substring( cp, 0 );
}

inline const TSubString string::substring( const char _FAR *cp ) const
    THROW_NONE
{
    return substring( cp, 0 );
}

inline size_t string::find_first_of( const string _FAR &s ) const THROW_NONE
{
    return find_first_of( s, 0 );
}

inline size_t string::find_first_not_of( const string _FAR &s ) const THROW_NONE
{
    return find_first_not_of( s, 0 );
}

inline size_t string::find_last_of( const string _FAR &s ) const THROW_NONE
{
    return find_last_of( s, NPOS );
}

inline size_t string::find_last_not_of( const string _FAR &s ) const THROW_NONE
{
    return find_last_not_of( s, NPOS );
}

inline int string::get_case_sensitive_flag()
{
    return case_sensitive;
}

inline int string::get_paranoid_check_flag()
{
    return paranoid_check;
}

inline int string::get_skip_whitespace_flag()
{
    return skip_white;
}

inline size_t string::get_initial_capacity()
{
    return initial_capac;
}

inline size_t string::get_resize_increment()
{
    return resize_inc;
}

inline size_t string::get_max_waste()
{
    return freeboard;
}

inline int string::is_null() const
{
    return *p->array==0;
}

#if defined( _Windows )
inline void string::ansi_to_oem() THROW_NONE
{
    ::AnsiToOem( p->array, p->array );
}

inline void string::oem_to_ansi() THROW_NONE
{
    ::OemToAnsi( p->array, p->array );
}
#endif

// Check to make sure a string index refers to a valid element
inline int string::valid_element( size_t n ) const THROW_NONE
{
    return n < length();
}

// Check to make sure a string index is in range
inline int string::valid_index( size_t n ) const THROW_NONE
{
    return n <= length();
}

// Constructor for internal use only
inline string::string( const string _FAR &s, const char _FAR *cp )
{
    p = new TStringRef( s.c_str(), s.length(), cp, cp?strlen(cp):0, 0 );
}

inline string _Cdecl operator + ( const string _FAR &s,
                                  const char _FAR *cp )
    THROW_XALLOC_LENGTHERROR
{
    return string(s,cp);
}

inline string _Cdecl operator + ( const string _FAR &s1,
                                  const string _FAR &s2 )
    THROW_XALLOC_LENGTHERROR
{
    return s1 + s2.c_str();
}

/*------------------------------------------------------------------------*/
/*                                                                        */
/*  TSubString inlines                                                    */
/*                                                                        */
/*------------------------------------------------------------------------*/

inline int TSubString::operator == ( const string _FAR &s ) const THROW_NONE
{
    return operator==(s.c_str());
}

inline int TSubString::operator != ( const char _FAR *cp ) const THROW_NONE
{
    return !operator==(cp);
}

inline int TSubString::operator != ( const string _FAR &s ) const THROW_NONE
{
    return !operator==(s.c_str());
}

inline char TSubString::get_at( size_t pos ) const THROW_OUTOFRANGE
{
    return (*this)[pos];
}

inline void TSubString::put_at( size_t pos, char c ) THROW_OUTOFRANGE
{
    (*this)[pos] = c;
}

inline char _FAR &TSubString::operator[]( size_t pos ) THROW_OUTOFRANGE
{
    return (*this)(pos);    // use operator()
}

inline char TSubString::operator[]( size_t pos ) const
    THROW_OUTOFRANGE
{
    assert_element(pos);
    return s->p->array[begin+pos];
}

inline char TSubString::operator()( size_t pos ) const
    THROW_OUTOFRANGE
{
#if defined( BOUNDS_CHECK )
    assert_element(pos);
#endif
    return s->p->array[begin+pos];
}

inline size_t TSubString::length() const THROW_NONE
{
    return extent;
}

inline int TSubString::start() const THROW_NONE
{
    return begin;
}

inline int TSubString::is_null() const THROW_NONE
{
    return begin == NPOS;
}

inline int TSubString::operator!() const THROW_NONE
{
    return begin == NPOS;
}

inline int TSubString::valid_element( size_t n ) const THROW_NONE
{
    return n < length();
}

// Private constructor
inline TSubString::TSubString(const string _FAR *sp,
                              size_t start,
                              size_t len ) :
    begin(start),
    extent(len),
    s((string _FAR *)sp)
{
}

inline TSubString string::operator()( const TRegexp _FAR & r ) THROW_NONE
{
    return (*this)(r,0);
}

inline const TSubString string::operator()( const TRegexp _FAR &r ) const THROW_NONE
{
    return (*this)(r,0);
}

#endif  // __CSTRING_H

