/*  except.h

    Definitions for exception handling

*/

/*
 *      C/C++ Run Time Library - Version 6.0
 *
 *      Copyright (c) 1992, 1993 by Borland International
 *      All Rights Reserved.
 *
 */

#ifndef __EXCEPT_H
#define __EXCEPT_H

#if !defined(___DEFS_H)
#include <_defs.h>
#endif
#if !defined(___STDLIB_H)
#include <stdlib.h>
#endif

#ifdef __cplusplus

typedef void (_RTLENTRY *terminate_function)();
typedef void (_RTLENTRY *unexpected_function)();

terminate_function  _RTLENTRY set_terminate(terminate_function);
unexpected_function _RTLENTRY set_unexpected(unexpected_function);

terminate_function  _RTLENTRY terminate();
unexpected_function _RTLENTRY unexpected();

extern  char *      _RTLENTRY __ThrowFileName();
extern  unsigned    _RTLENTRY __ThrowLineNumber();
extern  char *      _RTLENTRY __ThrowExceptionName();

#define  __throwFileName      __ThrowFileName()
#define  __throwLineNumber __ThrowLineNumber()
#define  __throwExceptionName __ThrowExceptionName()

/* Predefined exceptions */

/* Note:  The xmsg class definition is defined by ANSI in the draft
 *        document: X3J16/91 WG21/N0059.  Please do not change this
 *        definition, despite it's different style, unless the ANSI
 *        definition changes.
 */

class _EXPCLASS string;

class _EXPCLASS xmsg
{
public:
    xmsg(const string& msg);
    xmsg(const xmsg& msg);
    ~xmsg();

    const string& why() const;
    void raise() throw(xmsg);
    xmsg& operator=(const xmsg _FAR &src);
private:
    string *str;
};

inline const string& xmsg::why() const
{
    return *str;
};

/* Note:  The xalloc class definition is defined by ANSI in the draft
 *        document: X3J16/91 WG21/N0059.  Please do not change this
 *        definition, despite it's different style, unless the ANSI
 *        definition changes.
 */

class _EXPCLASS xalloc : public xmsg
{
public:
    xalloc(const string &msg, size_t size);

    size_t requested() const;
    void raise() throw(xalloc);
private:
    size_t siz;
};

inline size_t xalloc::requested() const
{
    return siz;
}

extern xalloc __xalloc;

#endif  // __cplusplus

#endif  // __EXCEPT_H
